import os


SNDTIMEO = os.getenv('ZMQ_SNDTIMEO', '30000')
SNDTIMEO = int(SNDTIMEO)
RCVTIMEO = os.getenv('ZMQ_RCVTIMEO', '30000')
RCVTIMEO = int(RCVTIMEO)

LINGER = 3
ENCODING_DEFAULT = "utf-8"

SEARCH_LIMIT = 100
FIREBALL_SIZE = os.getenv('CIFSDK_CLIENT_ZMQ_FIREBALL_SIZE', 500)
FIREBALL_SIZE = int(FIREBALL_SIZE)
TRACE = os.getenv('CIFSDK_CLIENT_ZMQ_TRACE')
